@@ -1,3 +1,5 @@
+import os
+
 from inquirer_rhy.prompt import prompt
 import sys
 import json
@@ -11,11 +13,33 @@ else:
     dir_char = '/'
 
 
-def _ask(question: dict):
+def _ask(question: dict, timeout: int = 0):
+    if timeout:
+        def ask():
+            def handle(signum, frame):
+                raise RuntimeError
+
+            import signal
+
+            try:
+                signal.signal(signal.SIGALRM, handle)
+                signal.alarm(timeout)
+                res = prompt(question)
+                signal.alarm(0)
+                return res
+            except RuntimeError:
+                if dir_char == '/':
+                    os.system('stty echo')
+                QproDefaultConsole.print('\n[bold yellow][Warning | 警告][/bold yellow]',
+                                         f"Time out & Return | 超时并返回: {question['default'] if 'default' in question else None}")
+                return question['default'] if 'default' in question else None
+    else:
+        def ask():
+            return prompt(question)[question['name']]
     try:
         if 'name' not in question:
             question['name'] = 'NoName'
-        return prompt(question)[question['name']]
+        return ask()
     except:
         exit(0)
 
